@@ -1,22 +1,22 @@
-from pathlib import Path
-from csv import DictReader
-from contextlib import contextmanager
 import os
+from contextlib import contextmanager, ExitStack
+from csv import DictReader
+from pathlib import Path
 from random import shuffle
 
-from sqlalchemy import create_engine, MetaData
-from sqlalchemy.sql.expression import func, select
+from sqlalchemy import MetaData, create_engine
 from sqlalchemy.sql.elements import quoted_name
-
+from sqlalchemy.sql.expression import func, select
 from yaml.representer import Representer
-from snowfakery.data_gen_exceptions import DataGenNameError
 
+from snowfakery.data_gen_exceptions import DataGenError, DataGenNameError
 from snowfakery.plugins import (
-    SnowfakeryPlugin,
     PluginResult,
     PluginResultIterator,
+    SnowfakeryPlugin,
     memorable,
 )
+from snowfakery.utils.files import FileLike, open_file_like
 from snowfakery.utils.yaml_utils import SnowfakeryDumper
 
 
@@ -28,7 +28,7 @@ def _open_db(db_url):
     return engine, metadata
 
 
-def sql_dataset(db_url: str, tablename: str = None, mode="linear"):
+def sql_dataset(db_url: str, tablename: str = None, mode="linear", repeat: bool = True):
     "Open the right SQL Dataset iterator based on the params"
     assert db_url
     engine, metadata = _open_db(db_url)
@@ -50,9 +50,9 @@ def sql_dataset(db_url: str, tablename: str = None, mode="linear"):
             f"Database has multiple tables in it and none was selected: {metadata.tables.keys()}"
         )
     if mode == "linear":
-        return SQLDatasetLinearIterator(engine, table)
+        return SQLDatasetLinearIterator(engine, table, repeat)
     elif mode == "shuffle":
-        return SQLDatasetRandomPermutationIterator(engine, table)
+        return SQLDatasetRandomPermutationIterator(engine, table, repeat)
     else:
         raise NotImplementedError(f"Unknown mode: {mode}")
 
@@ -63,30 +63,15 @@ class DatasetIteratorBase(PluginResultIterator):
     Subclasses should implement 'self.restart' which puts an iterator into 'self.results'
     """
 
-    def next(self):
-        try:
-            return next(self.results)
-        except StopIteration:
-            self.restart()
-            return next(self.results)
-
-    def start(self):
-        "Initialize the iterator in self.results."
-        raise NotImplementedError(f"start method on {self.__class__.__name__}")
-
-    def restart(self):
-        "Restart the iterator by assigning to self.results"
-        self.start()
-
-    def close(self):
-        "Subclasses should implement this if they need to clean up resources"
-        pass  # pragma: no cover
+    def next_result(self):
+        return next(self.results)
 
 
 class SQLDatasetIterator(DatasetIteratorBase):
-    def __init__(self, engine, table):
+    def __init__(self, engine, table, repeat):
         self.connection = engine.connect()
         self.table = table
+        super().__init__(repeat)
         self.start()
 
     def start(self):
@@ -119,19 +104,34 @@ class SQLDatasetRandomPermutationIterator(SQLDatasetIterator):
 
 
 class CSVDatasetLinearIterator(DatasetIteratorBase):
-    def __init__(self, datasource: Path):
-        self.datasource = datasource
-        self.file = open(self.datasource, newline="", encoding="utf-8-sig")
+    def __init__(self, datasource: FileLike, repeat: bool):
+        self.cleanup = ExitStack()
+        # utf-8-sig and newline="" are for Windows
+        self.path, self.file = self.cleanup.enter_context(
+            open_file_like(datasource, "r", newline="", encoding="utf-8-sig")
+        )
+
         self.start()
+        super().__init__(repeat)
 
     def start(self):
         self.file.seek(0)
         d = DictReader(self.file)
-        self.results = (DatasetPluginResult(row) for row in d)
+
+        plugin_result = self.plugin_result
+        self.results = (plugin_result(row) for row in d)
 
     def close(self):
         self.results = None
-        self.file.close()
+        self.cleanup.close()
+
+    def plugin_result(self, row):
+        if None in row:
+            raise DataGenError(
+                f"Your CSV row has more columns than the CSV header:  {row[None]}, {self.path} {self.file}"
+            )
+
+        return DatasetPluginResult(row)
 
 
 class DatasetPluginResult(PluginResult):
@@ -196,10 +196,11 @@ class FileDataset(DatasetBase):
     def _load_dataset(self, iteration_mode, rootpath, kwargs):
         dataset = kwargs.get("dataset")
         tablename = kwargs.get("table")
+        repeat = kwargs.get("repeat", True)
 
         with chdir(rootpath):
             if "://" in dataset:
-                return sql_dataset(dataset, tablename, iteration_mode)
+                return sql_dataset(dataset, tablename, iteration_mode, repeat)
             else:
                 filename = Path(dataset)
 
@@ -212,9 +213,9 @@ class FileDataset(DatasetBase):
                     )
 
                 if iteration_mode == "linear":
-                    return CSVDatasetLinearIterator(filename)
+                    return CSVDatasetLinearIterator(filename, repeat)
                 elif iteration_mode == "shuffle":
-                    return CSVDatasetRandomPermutationIterator(filename)
+                    return CSVDatasetRandomPermutationIterator(filename, repeat)
 
 
 class DatasetPluginBase(SnowfakeryPlugin):
