@@ -37,6 +37,9 @@ class StringGenerator:
     def __radd__(self, other):
         return str(other) + str(self)
 
+    def render(self):
+        return self.func()
+
 
 class FakerTemplateLibrary:
     """A Jinja template library to add the fake.xyz objects to templates"""
@@ -66,7 +69,7 @@ number_chars = set(string.digits + ".")
 
 def look_for_number(arg):
     looks_like_float = False
-    if len(arg) == 0:
+    if len(arg) == 0 or (arg[0] == "0" and arg[1:2] != "."):
         return arg
     for char in arg:
         if char not in number_chars:
